using ComputersShopBusinessLogic.BusinessLogics;
using ComputersShopContracts.BusinessLogicContracts;
using ComputersShopContracts.StoragesContracts;
using ComputersShopFileImplement.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;


namespace ComputersShopView
{
    internal static class Program
	{
		private static ServiceProvider? _serviceProvider;
		public static ServiceProvider? ServiceProvider => _serviceProvider;
		/// <summary>
		///  The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			// To customize application configuration such as set high DPI settings or default font,
			// see https://aka.ms/applicationconfiguration.
			ApplicationConfiguration.Initialize();
			var services = new ServiceCollection();
			ConfigureServices(services);
			_serviceProvider = services.BuildServiceProvider();
			Application.Run(_serviceProvider.GetRequiredService<FormMain>());
		}

		private static void ConfigureServices(ServiceCollection services)
		{
			services.AddLogging(option =>
			{
				option.SetMinimumLevel(LogLevel.Information);
				option.AddNLog("nlog.config");
			});
			services.AddTransient<IComponentStorage, ComponentStorage>();
			services.AddTransient<IComputerStorage, ComputerStorage>();
			services.AddTransient<IOrderStorage, OrderStorage>();

			services.AddTransient<IComponentLogic, ComponentLogic>();
			services.AddTransient<IComputerLogic, ComputerLogic>();
			services.AddTransient<IOrderLogic, OrderLogic>();

			services.AddTransient<FormMain>();
			services.AddTransient<FormComponent>();
			services.AddTransient<FormComponents>();
			services.AddTransient<FormCreateOrder>();
			services.AddTransient<FormComputer>();
			services.AddTransient<FormComputers>();
			services.AddTransient<FormComputerComponent>();
		}
	}
}